import os
import re
import sys
import importlib.util

from hape.logging import Logging
from hape.services.file_service import FileService

class Init:

    def __init__(self, name: str):
        self.logger = Logging.get_logger('hape.hape_cli.models.init_model')
        self.name = name
        self.name_underscore = name.replace("-", "_")
        self.file_service = FileService()
        
        spec = importlib.util.find_spec("hape")
        if spec and spec.origin:
            self.hape_framework_path = os.path.dirname(os.path.abspath(spec.origin))
        else:
            self.logger.error("Couldn't not find `hape` package. Execute `pip install --upgrade hape`.")
            exit(1)
        
        self.hape_files = [
            'artifacts/Makefile',
            'artifacts/alembic.ini',
            'artifacts/main.py',
            'artifacts/requirements.txt',
            'artifacts/setup.py'
        ]
        # self.hape_files = [
        #     "artifacts/.dockerignore",
        #     "artifacts/.env.example",
        #     "artifacts/.gitignore",
        #     "artifacts/alembic.ini",
        #     "artifacts/main.py",
        #     "artifacts/Makefile",
        #     "artifacts/requirements.txt",
        #     "artifacts/setup.py"
        # ]
        
        self.hape_dirs = []
        # self.hape_dirs = [
        #     "artifacts/dockerfiles",
        #     "artifacts/scripts",
        # ]

        self.PROJECT_STRUCTURE = {
            ".dockerignore": None,
            ".env.example": None,
            ".gitignore": None,
            "alembic.ini": None,
            "main.py": None,
            "Makefile": None,
            "README.md": None,
            "requirements-dev.txt": None,
            "setup.py": None,
            "dockerfiles": [],
            "scripts": [],
            self.name_underscore: {
                "argument_parsers": [".gitkeep"],
                "cli.py": None,
                "controllers": ["__init__.py"],
                "enums": ["__init__.py"],
                "migrations": ["README", "env.py", "script.py.mako"],
                "migrations/versions": [".gitkeep"],
                "models": ["__init__.py"],
                "services": ["__init__.py"],
            }
        }

    def __copy_hape_files(self):
        for file in self.hape_files:
            src_file = os.path.join(self.hape_framework_path, file)
            dest_file = os.path.join(self.name, file).replace('artifacts/', '')
            self.logger.debug(f'Copying file: {src_file} -> {dest_file}')
            self.file_service.copy_file(src_file, dest_file, overwrite=True)

        for directory in self.hape_dirs:
            src_dir = os.path.join(self.hape_framework_path, directory)
            dest_dir = os.path.join(self.name, directory)
            self.logger.debug(f'Copying directory: {src_dir} -> {dest_dir}')
            self.file_service.copy_directory(src_dir, dest_dir)

    def __init_project_structure_process_dictionary(self, dictionary: dict, root_path: str):
        for key, value in dictionary.items():
            sub_path = os.path.join(root_path, key)
            if value is None:
                self.logger.debug(f'Creating file: {sub_path}')
                self.file_service.write_file(sub_path, "")
            elif isinstance(value, str):
                self.logger.debug(f'Creating file: {sub_path}')
                self.file_service.write_file(sub_path, value)
            elif isinstance(value, list):
                self.logger.debug(f'Creating directory: {sub_path}')
                self.file_service.create_directory(sub_path)
                for file in value:
                    self.logger.debug(f'Creating file: {sub_path}')
                    self.file_service.write_file(os.path.join(sub_path, file), "")
            elif isinstance(value, dict):
                self.__init_project_structure_process_dictionary(value, sub_path)

    def __init_project_structure(self):
        self.logger.debug(f'Creating directory: {self.name}')
        if self.file_service.path_exists(self.name):
            self.logger.warning(f"Warning: directory '{self.name}' already exists.")
            user_input = input(f"ALL DATA IN '{self.name}' WILL BE LOST. Do you want to remove '{self.name}' and initialize a new project? yes/no: ").strip().lower()
            if user_input == 'yes':
                self.file_service.delete_directory(self.name)
            else:
                print("Operation canceled. Keeping the directory.")
                sys.exit(1)
        self.file_service.create_directory(self.name)
        self.__init_project_structure_process_dictionary(self.PROJECT_STRUCTURE, self.name)

        readme_path = os.path.join(self.name, "README.md")
        self.logger.debug(f'Creating file: {readme_path}')
        self.file_service.write_file(readme_path, f"# {self.name}\n")

    def validate(self):
        self.logger.debug(f"validate()")
        name = self.name.strip()
        self.logger.debug(f'Validating project name: {self.name}')
        if not re.match(r'^[a-z0-9]+(-[a-z0-9]+)*$', name):
            self.logger.error(f"Error: Project name '{name}' must contain only lowercase letters, numbers, and use '-' as a separator.")
            sys.exit(1)
        self.logger.debug(f'Valid project name.')

    def init_project(self):
        self.logger.debug(f"init_project()")
        self.__init_project_structure()
        self.__copy_hape_files()
        self.logger.info(f'Project {self.name} has been successfully initialized!')



from hape.logging import Logging

import re
from typing import List, Optional
from datetime import datetime
from gitlab import Gitlab
from gitlab.v4.objects import Group, Project, GroupProject, ProjectCommit
from hape.config import Config
from hape.services.file_service import FileService

class GitlabService:
    def __init__(self):
        self.logger = Logging.get_logger('hape.services.gitlab_service')
        gitlab_domain = Config.get_gitlab_domain()
        gitlab_token = Config.get_gitlab_token()
        self.client = Gitlab(f"https://{gitlab_domain}", private_token=gitlab_token)
        self._per_page = 100
    
    def _get_project(self, id: int) -> Project:
        return self.client.projects.get(id)
    
    def _get_projects(self, ids: list[int]) -> List[Project]:
        return [self.client.projects.get(id) for id in ids]
    
    def _get_group_projects(self, group_id: int, recursive: bool = False, search: Optional[str] = None) -> List[GroupProject]:
        group: Group = self.client.groups.get(group_id)
        return group.projects.list(get_all=True, per_page=self._per_page, include_subgroups=recursive, search=search)        
    
    def _get_project_commits(self, project: Project, start_date: Optional[datetime] = None, end_date: Optional[datetime] = None) -> List[ProjectCommit]:
        params = {"get_all": True}
        if start_date:
            params["since"] = start_date.isoformat()
        if end_date:
            params["until"] = end_date.isoformat()
        
        return project.commits.list(**params)
    
    def generate_csv_changes_in_cicd_repos(self, group_id: int, output_file: str, start_date: Optional[datetime] = None, end_date: Optional[datetime] = None, file_regex: Optional[str] = None):
        self.logger.debug(f"generate_csv_changes_in_cicd_repos(group_id: int: {group_id: int}, output_file: str: {output_file: str}, start_date: Optional[datetime] : {start_date: Optional[datetime] }, end_date: Optional[datetime] : {end_date: Optional[datetime] }, file_regex: Optional[str] : {file_regex: Optional[str] })")
        
        group_projects = self._get_group_projects(group_id, recursive=True, search="cicd-")
        project_ids = [project.id for project in group_projects]
        projects = self._get_projects(project_ids)

        all_commits = []
        for project in projects:
            project_path = project.path.replace("cicd-", "")
            commits = self._get_project_commits(project, start_date, end_date)
            for commit in commits:
                if commit.author_name == f"{project_path}-ci":
                    continue

                diff = commit.diff()
                for change in diff:
                    if file_regex and re.match(file_regex, change['new_path']):
                        all_commits.append({
                            "project_name": project.name,
                            "commit_date": commit.committed_date,
                            "commit_author": commit.author_name,
                            "commit": commit.id,
                            "web_url": commit.web_url
                        })

        FileService().write_csv_file(output_file, all_commits)
        print(f"CSV report generated: {output_file}")



import os
import json
from dotenv import load_dotenv
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.exc import OperationalError
from sqlalchemy.sql import text
from hape.logging import Logging

class Config:
    logger = Logging.get_logger('hape.config')
    _env_loaded = False
    _db_session = None
    _required_env_variables = []

    @staticmethod
    def check_variables():
        Config.logger.debug(f"check_variables()")
        for variable in Config._required_env_variables:
            Config._get_env_value(variable)

    @staticmethod
    def _load_environment():
        if not Config._env_loaded:
            if os.path.exists(".env"):
                load_dotenv()
            Config._env_loaded = True

    @staticmethod
    def _get_env_value(env):
        Config._load_environment()
        env_value = os.getenv(env)
        
        if not env_value and env in Config._required_env_variables:
            Config.logger.error(f"""Environment variable {env} is missing.

To set the value of the environment variable run:
$ export {env}="value"

The following environment variables are required:
{json.dumps(Config._required_env_variables, indent=4)}
""")
            exit(1)
        return env_value

    @staticmethod
    def get_db_url():
        Config.logger.debug(f"get_db_url()")
        return f"mysql+pymysql://{Config.get_mysql_username()}:{Config.get_mysql_password()}@{Config.get_mysql_host()}/{Config.get_mysql_database()}"

    @staticmethod
    def get_db_session() -> sessionmaker:
        Config.logger.debug(f"get_db_url()")
        if not Config._db_session:
            try:
                DATABASE_URL = Config.get_db_url()
                engine = create_engine(DATABASE_URL, echo=True)
                with engine.connect() as connection:
                    connection.execute(text("SELECT 1"))

                Config._db_session = sessionmaker(autocommit=False, autoflush=False, bind=engine)
                Config.logger.info("Database seassion created successfully.")
            except OperationalError:
                Config.logger.error("Error: Unable to connect to the database. Please check the configuration.")
                raise

        return Config._db_session()

    @staticmethod
    def get_gitlab_token():
        Config.logger.debug(f"get_gitlab_token()")
        return Config._get_env_value("HAPE_GITLAB_TOKEN")
    
    @staticmethod
    def get_gitlab_domain():
        Config.logger.debug(f"get_gitlab_domain()")
        return Config._get_env_value("HAPE_GITLAB_DOMAIN")
    
    @staticmethod
    def get_mysql_host():
        Config.logger.debug(f"get_mysql_host()")
        return Config._get_env_value("HAPE_MARIADB_HOST")

    @staticmethod
    def get_mysql_username():
        Config.logger.debug(f"get_mysql_username()")
        return Config._get_env_value("HAPE_MARIADB_USERNAME")

    @staticmethod
    def get_mysql_password():
        Config.logger.debug(f"get_mysql_password()")
        return Config._get_env_value("HAPE_MARIADB_PASSWORD")

    @staticmethod
    def get_mysql_database():
        Config.logger.debug(f"get_mysql_database()")
        return Config._get_env_value("HAPE_MARIADB_DATABASE")
    
    @staticmethod
    def get_log_level():
        Config.logger.debug(f"get_log_level()")
        valid_levels = ['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']
        log_level = Config._get_env_value("HAPE_LOG_LEVEL")
        return log_level if log_level and log_level in valid_levels else "DEBUG"
    
    @staticmethod
    def get_log_file():
        Config.logger.debug(f"get_log_file()")
        log_file = Config._get_env_value("HAPE_LOG_FILE")
        return log_file if log_file else "hape.log"
    
    @staticmethod
    def get_log_rotate_every_run():
        Config.logger.debug(f"get_log_rotate_every_run()")
        log_file = Config._get_env_value("HAPE_LOG_ROTATE_EVERY_RUN")
        return log_file if log_file else "1"
